import { checkContext } from "feathers-hooks-common";
import { getItemsIsArray } from "../utils/getItemsIsArray";
export function removeRelated({ service, keyThere, keyHere = "id", blocking = true }) {
    if (!service || !keyThere) {
        throw "initialize hook 'removeRelated' completely!";
    }
    return async (context) => {
        // @ts-expect-error wait for feathers-hooks-common to update
        checkContext(context, "after", "remove", "removeRelated");
        const { items } = getItemsIsArray(context);
        let ids = items.map(x => x[keyHere]).filter(x => !!x);
        ids = [...new Set(ids)];
        if (!ids || ids.length <= 0) {
            return context;
        }
        // feathers does not accept `paginate: false` for remove, but some adapters need it to work properly
        const promise = context.app.service(service).remove(null, {
            query: {
                [keyThere]: {
                    $in: ids
                }
            },
            paginate: false
        });
        if (blocking) {
            await promise;
        }
        return context;
    };
}
