"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeRelated = void 0;
const feathers_hooks_common_1 = require("feathers-hooks-common");
const getItemsIsArray_1 = require("../utils/getItemsIsArray");
function removeRelated({ service, keyThere, keyHere = "id", blocking = true }) {
    if (!service || !keyThere) {
        throw "initialize hook 'removeRelated' completely!";
    }
    return (context) => __awaiter(this, void 0, void 0, function* () {
        // @ts-expect-error wait for feathers-hooks-common to update
        (0, feathers_hooks_common_1.checkContext)(context, "after", "remove", "removeRelated");
        const { items } = (0, getItemsIsArray_1.getItemsIsArray)(context);
        let ids = items.map(x => x[keyHere]).filter(x => !!x);
        ids = [...new Set(ids)];
        if (!ids || ids.length <= 0) {
            return context;
        }
        // feathers does not accept `paginate: false` for remove, but some adapters need it to work properly
        const promise = context.app.service(service).remove(null, {
            query: {
                [keyThere]: {
                    $in: ids
                }
            },
            paginate: false
        });
        if (blocking) {
            yield promise;
        }
        return context;
    });
}
exports.removeRelated = removeRelated;
